"""An implementation of an Alias that works in the Virtual Host Folder"""


# standard issue modules
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_parent, aq_base
import re

# my own

from Products.Fakers import ZFakeFolder, ZFakeProduct
import IHost, IAlias, utils

#bit map settings
DESTINATION_MASK=7
DESTINATION_SHOWN_HIDDEN_MASK=5
DESTINATION_SHOWN_LAST=7
DESTINATION_SHOWN_FIRST=5
DESTINATION_HIDDEN_LAST=3
DESTINATION_HIDDEN_FIRST=1
# 0            0          0
# hidden/shown first/last off/on
FORCE_ALIAS_MASK=24
FORCE_ACTIVE_ALIAS=24
FORCE_PASSIVE_ALIAS=8
# 0              0      (0 0 0)
# passive/active off/on
INCLUDE_PARENTS_MASK=32
INCLUDE_PARENTS=32
# 0      (0 0) (0 0 0)
# off/on
INCLUDE_HOST_ROOT_MASK=64
INCLUDE_HOST_ROOT=64
# 0      (0) (0 0) (0 0 0)
# off/on
RETAIN_PATH_AFTER_ALIAS_MASK=128
RETAIN_PATH_AFTER_ALIAS=128
# 0      (0) (0) (0 0) (0 0 0)
# off/on

SIMPLE_SET_MASK = DESTINATION_MASK + FORCE_ALIAS_MASK + INCLUDE_PARENTS_MASK + INCLUDE_HOST_ROOT_MASK

IS_ACTIVE_MASK=1+FORCE_ACTIVE_ALIAS # that is, destination is on, forcing active alias
INTERPRETATION_MASK=FORCE_ALIAS_MASK+INCLUDE_HOST_ROOT_MASK+DESTINATION_SHOWN_HIDDEN_MASK+INCLUDE_PARENTS_MASK
INTERPRETATION_DICT={
    FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST+INCLUDE_PARENTS:
    "Alias appears to be an actual folder, with acquisition enabled for the paths both to the destination folder and to the alias parent location",
    #FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST:
    #"Alias is in use, attempting to present alias as an actual folder, with acquisition enabled for the paths both to the destination folder and to the virtual host root location",
    FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+INCLUDE_PARENTS:
    "Alias appears to be an actual folder, acquired by the path to the alias parent location (and not the actual destination path)",
    #FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST:
    #"Alias is in use, attempting to present alias as an actual folder, acquired by the path to the virtual host root location (and not the actual destination path)",
    DESTINATION_SHOWN_FIRST:
    "Alias makes no pretense to be anything other than a temporary redirect (useful for images, etc.), without any connection to the virtual host root or parents",
    INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST:
    "Alias uses standard acquisition to find the destination path so that both the virtual host root and the full path to the destination are available",
    INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST+INCLUDE_PARENTS:
    "Alias uses standard acquisition to find the destination path so that both the path to the alias parents and the full path to the destination are available"
    }

class Alias(ZFakeFolder.ZFakeFolder, ZFakeProduct.ZFakeProduct):
    """Represents one alias, an optional part of a virtual host.  Holds alias
    information, and can act as a dynamically-inserted pseudo-folder in
    traversal."""

    __implements__= IAlias.IAliasHostOverrideContainer, IAlias.IAliasHostOverride, IAlias.IHostOverride

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    meta_type="Virtual Host Alias"
    icon="misc_/VirtualHostFolder/alias"

    manage_options=(
        (
        {'label':'Configuration', 'action':'manage_main',
        'help':('VirtualHostFolder','Alias_Configure.stx')},
        )
        +ZFakeFolder.ZFakeFolder.manage_options
        )

    id=parent=mapping=""
    includeDestinationPath=forceAlias=includeParents=includeHostRoot=0

    def __init__(self, id, title=""):
        """an init"""
        ZFakeFolder.ZFakeFolder.__init__(self, id, title)
        self.aliasName=""
        self.parent=self.mapping="/"
        self.settingsBitmap=0
        self.setRetainPathAfterAlias(1)
        self.setAddableClasses(Alias)
        self.showMoreOptions=0

    ## and now we need UI methods

    security.declareProtected( 'View management screens', 'manage_main' )
    manage_main = DTMLFile('dtml/AliasManage_main', globals())

    security.declareProtected( 'Manage properties', 'manage_alias' )
    def manage_alias(self, title="", aliasName="", parent="", mapping="", includeDestinationPath=0, includeParents=0, includeHostRoot=0, forceAlias=0, retainPathAfterAlias=0, simple=0, toggleShowMoreOptions=0, REQUEST=None):
        """The basic editing function that manage_main calls"""
        messages=[]
        mfunc=messages.append
        if toggleShowMoreOptions:
            self.showMoreOptions = not (self.showMoreOptions)
        else:
            ret=0
            ret+=self.setTitle(title)
            if self.showMoreOptions:
                ret+=self.setIncludeDestinationPath(includeDestinationPath, mfunc)
                ret+=self.setForceAlias(forceAlias, mfunc)
                ret+=self.setIncludeParents(includeParents, mfunc)
                ret+=self.setIncludeHostRoot(includeHostRoot, mfunc)
            else:
                ret+=self.setSimple(simple, mfunc)
            ret+=self.setRetainPathAfterAlias(retainPathAfterAlias, mfunc)
            ret+=self.setMapping(mapping, mfunc)
            ret+=self.setParentsAndAliasName(parent, aliasName, mfunc)
            if not ret: mfunc("No changes made")
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, messages=messages)
        return messages

    security.declarePublic( 'getIsTopLevel' )
    def getIsTopLevel(self):
        """in normal traversal, determines if this Alias is immediately beneath
        a host object or if it is a child of another alias.  Used by the
        manage_main DTML function"""
        return IHost.IHost.isImplementedBy(aq_parent(self))

    security.declareProtected( 'Manage properties', 'setSimple' )
    def setSimple(self, setting, mfunc=None):
        """sets flags wholesale"""
        currSetting=self.settingsBitmap & SIMPLE_SET_MASK
        if currSetting==setting: return 0
        self.settingsBitmap=self.settingsBitmap-currSetting+setting
        self.setFull()
        return 1
    security.declarePublic( 'getSimple' )
    def getSimple(self):
        """gets the current state of the flags that setSimple messes with"""
        return self.settingsBitmap & SIMPLE_SET_MASK
    security.declareProtected( 'Manage properties', 'setIncludeDestinationPath' )
    def setIncludeDestinationPath(self, setting, mfunc=None):
        """Sets the include destination path flags (see interface for more)"""
        if mfunc and setting==self.getIncludeDestinationPath(): return 0
        if setting not in (0,DESTINATION_SHOWN_LAST, DESTINATION_SHOWN_FIRST, DESTINATION_HIDDEN_LAST, DESTINATION_HIDDEN_FIRST):
            if mfunc:
                mfunc("Not a valid setting for 'Include Destination Path'")
            return 0
        self.settingsBitmap=(self.settingsBitmap&(~DESTINATION_MASK))+setting
        return 1
    security.declarePublic( 'getIncludeDestinationPath' )
    def getIncludeDestinationPath(self):
        """gets the include destination path flags"""
        return self.settingsBitmap&DESTINATION_MASK
    security.declareProtected( 'Manage properties', 'setForceAlias' )
    def setForceAlias(self, setting, mfunc=None):
        """alias object can be forced in path in an active or passive
        manner.  Setting is here"""
        if mfunc and setting==self.getForceAlias(): return 0
        if setting not in (0,FORCE_ACTIVE_ALIAS, FORCE_PASSIVE_ALIAS):
            if mfunc:
                mfunc("Not a valid setting for 'forceAlias'")
            return 0
        self.settingsBitmap=(self.settingsBitmap&(~FORCE_ALIAS_MASK))+setting
        self.setFull()
        return 1
    security.declarePublic( 'getForceAlias' )
    def getForceAlias(self):
        """get whether the alias object is forced in the path (and the manner
        of the forcing)"""
        return self.settingsBitmap&FORCE_ALIAS_MASK
    security.declareProtected( 'Manage properties', 'setIncludeParents' )
    def setIncludeParents(self, setting, mfunc=None):
        """sets whether or not the parents of the alias will actually be
        included in the path"""
        setting=setting and INCLUDE_PARENTS or 0
        if mfunc and setting==self.getIncludeParents(): return 0
        self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_PARENTS_MASK))+setting
        if setting and not self.getIncludeHostRoot():
            self.setIncludeHostRoot(1, mfunc)
            if mfunc: mfunc("Including the alias parents in the acquisition path requires that the host root be included, so that setting has been changed as well")
        return 1
    security.declarePublic( 'getIncludeParents' )
    def getIncludeParents(self):
        """gets includeParents setting"""
        return self.settingsBitmap&INCLUDE_PARENTS_MASK
    security.declareProtected( 'Manage properties', 'setIncludeHostRoot' )
    def setIncludeHostRoot(self, setting, mfunc=None):
        """sets whether or not the host root will be included in the
        acquisition path"""
        setting=setting and INCLUDE_HOST_ROOT or 0
        if mfunc and setting==self.getIncludeHostRoot(): return 0
        if self.getIncludeParents():
            self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_HOST_ROOT_MASK))+INCLUDE_HOST_ROOT
            return setting==INCLUDE_HOST_ROOT
        self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_HOST_ROOT_MASK))+setting
        return 1
    security.declarePublic( 'getIncludeHostRoot' )
    def getIncludeHostRoot(self):
        """gets includeHostRoot setting"""
        return self.settingsBitmap&INCLUDE_HOST_ROOT_MASK

    security.declareProtected( 'Manage properties', 'setRetainPathAfterAlias' )
    def setRetainPathAfterAlias(self, setting, mfunc=None):
        """Sets whether the path after alias should be retained in the
        stack.  If it is not retained, the alias' index_html will
        always be returned for any call down this path.  Any
        boolean-equivalent true/false setting will work.  If mfunc is
        included, it may be called with explanatory messages (usually
        mfunc is an array append)."""
        setting=setting and RETAIN_PATH_AFTER_ALIAS or 0
        if mfunc and setting==self.getRetainPathAfterAlias(): return 0
        self.settingsBitmap=(self.settingsBitmap&(~RETAIN_PATH_AFTER_ALIAS_MASK))+setting
        return 1

    security.declarePublic( 'getRetainPathAfterAlias' )
    def getRetainPathAfterAlias(self):
        """Gets the setting described above, returning either 0 (off)
        or a positive integer (on)."""
        return self.settingsBitmap & RETAIN_PATH_AFTER_ALIAS_MASK

    security.declareProtected( 'Manage properties', 'setParentsAndAliasName' )
    def setParentsAndAliasName(self, parent, aliasName, mfunc=None):
        """parents and alias name are set together here so that the request can
        be checked intelligently"""
        if self.getForceAlias():
            aliasName=self.getAliasName()
        else:
            aliasName=aliasName.strip()
        parent=parent.strip()
        if not len(parent) or parent[0] != "/":
            parent="/"+parent
        parent=utils.cleanPath.match(parent).group(1)
        if mfunc and aliasName==self.getAliasName() and parent==self.getParent(): return 0
        if not self.getForceAlias() and not utils.isAFolderName.match(aliasName):
            if mfunc: mfunc("Alias name %s cannot be used.  Alias names must begin with a letter and be at least one character long." % aliasName)
            return 0
        if not aq_parent(self).checkParentsAndAliasName(parent, aliasName, self, mfunc):
            return 0
        if not self.getForceAlias():
            self.aliasName=aliasName
        self.parent=parent
        self.setFull()
        return 1
    security.declareProtected( 'View management screens', 'checkParentsAndAliasName' )
    def checkParentsAndAliasName(self, parent, aliasName, caller, mfunc=None):
        """A helper function for setParentsAndAliasName: a function of this
        name will be called in the parent of the given alias"""
        if parent and parent!="/":
            if mfunc: mfunc("An alias within an alias may not specify any parent: it acts as if it resides within the parent alias folder.")
            return 0
        obj= self.getItemById(aliasName)
        if obj and obj!=caller:
            if mfunc: mfunc("An alias of this name already is defined in the parent alias")
            return 0
        return 1
    security.declarePublic( 'getMapping' )
    def getMapping(self):
        """get the destination mapping"""
        return self.mapping
        # parent and mapping should begin with a / but not end with one
    security.declarePublic( 'getAliasName' )
    def getAliasName(self):
        """get the alias name"""
        if self.getForceAlias():
            return self.getId()
        return self.aliasName
    security.declarePublic( 'getTrueAliasName' )
    def getTrueAliasName(self):
        """get the alias name"""
        return self.aliasName
    security.declarePublic( 'getParent' )
    def getParent(self):
        """get the parent"""
        return self.parent
    security.declarePrivate( 'setFull' )
    def setFull(self, mfunc=None):
        """set the convenience data fullURL and fullPath"""
        self.fullURL=self.getParent() or "/"
        self.fullURL+=(self.fullURL[-1]!="/" and "/" or "") +self.getAliasName()
        self.fullPath=self.fullURL.split("/")[1:]
        return 1
    security.declarePublic( 'getFullURL' )
    def getFullURL(self):
        """get the convenience data fullURL (parent plus aliasName)"""
        return self.fullURL
    security.declarePublic( 'getFullPath' )
    def getFullPath(self):
        """get the convenience data fullPath (list version of fullURL)"""
        return self.fullPath
    security.declarePrivate( 'setMapping' )
    def setMapping(self, mapping, mfunc=None):
        """set the destination mapping"""
        if mfunc and mapping==self.getMapping(): return 0
        mapping=mapping.strip()
        if not len(mapping) or mapping[0] != "/":
            mapping="/"+mapping
        mapping=utils.cleanPath.match(mapping).group(1)
        if not self._check_path(mapping):
            if mfunc:"mapping (destination) folder not found or not a folder."
            return 0
        self.mapping=mapping
        self.mappingPath=self.mapping.split("/")[1:]
        return 1
    security.declarePublic( 'getStatus' )
    def getStatus(self, mfunc=None):
        """get the status of the alias, on(bitmap) or off(0).  The bitmap
        contains the setting data for this alias.  If mfunc is included then it
        is called with an interpretation of the current settings."""
        ret=((self.settingsBitmap & IS_ACTIVE_MASK) in (IS_ACTIVE_MASK, 1, FORCE_ACTIVE_ALIAS, 1+FORCE_PASSIVE_ALIAS)) and self.settingsBitmap # or 0
        if not mfunc:
            return ret
        if not ret:
            mfunc("Alias is not in use.  To activate, force alias to be active or include the destination path somewhere in the acquisition path")
            return ret # that is, 0
        ret=ret&INTERPRETATION_MASK
        if INTERPRETATION_DICT.has_key(ret):
            mfunc(INTERPRETATION_DICT[ret])
        else:
            mfunc("Alias is using an unusual setting.  For more information, see the help link")
        if not self.getRetainPathAfterAlias():
            mfunc("Alias will delete all path information after the alias itself: all URLs that have this full URL (parents plus alias name) as their base will be delivered to the index_html method of the destination folder object")
        return self.settingsBitmap


    # so stack needs to have path from final object to just before alias destination, "//hostName/aliasName/childAliasName", "VHF", path from alias to virtual root, "/", "VHF", possibly optional path from virtual root to zope root, optional path from destination folder to zope root (may also precede virtual-to-zope-root path instead)
    # This means host needs to return optional alias name string, optional path from alias to virtual root, hidden path.

    security.declarePublic( 'testOverride' )
    def testOverride(self, host, stack, stackString=""):
        """tries to quickly determine if this alias should be used--only called
        in top-level alias objects."""
        if not self.getStatus(): return 0
        return self.fullURL==stackString or self.fullURL+"/"==stackString[:len(self.fullURL)+1]

    security.declarePublic( 'activateOverride' )
    def activateOverride(self, stack, VHFid, request, path, addressList, matchedStack=[]):
        """Setting the stack as needed--most of the hard work, other than
        __bobo_traverse__"""
        addressList.append(self.getId())
        aliasPath=self.fullPath[:]
        for onePath in aliasPath:
            matchedStack.append(stack.pop())
            assert matchedStack[-1]==onePath, "error in alias set stack--match reported but not found"
        kids=self.getAcquiredItems()
        if not self.getIncludeParents() and stack:
            kids=self.getAcquiredItems()
            if kids:
                for child in kids:
                    if child.getId()==stack[-1]:
                        if child.getStatus():
                            return child.activateOverride(stack, VHFid, request, path, addressList, matchedStack)
                        else: break

        # If a child, passive alias can't be allowed (or else parent
        # aliases will be searched for next item)

        vhRootString=""
        addressString="/".join(addressList)
        destPath=self.mappingPath[:]
        # destPath.reverse()

        fa=self.getForceAlias()
        idp=self.getIncludeDestinationPath()
        ip=self.getIncludeParents()
        ihr=self.getIncludeHostRoot()
        rpaa=self.getRetainPathAfterAlias()

        if not rpaa:
            while stack:
                stack.pop()
        if fa: # not self.includeDestinationPath or
            stack.append("//"+addressString)
            stack.append(VHFid)
            if fa==FORCE_ACTIVE_ALIAS:
                destPath.pop()
        if idp==DESTINATION_SHOWN_LAST:
            for i in range(len(destPath)):
                stack.append(destPath.pop())
        if ip:
            #matchedStack.reverse()
            #stack.extend(matchedStack)
            matchedStack.pop()
            for i in range(len(matchedStack)):
                stack.append(matchedStack.pop())
        if idp==DESTINATION_SHOWN_FIRST:
            for i in range(len(destPath)):
                stack.append(destPath.pop())
        if idp==DESTINATION_HIDDEN_LAST or ihr or ip or idp==DESTINATION_HIDDEN_FIRST:
            stack.append("/")
            stack.append(VHFid)
            if idp==DESTINATION_HIDDEN_LAST:
                for i in range(len(destPath)):
                    stack.append(destPath.pop())
                vhRootString=self.mapping+vhRootString
            if ihr or ip: # include Virtual Root
                list_path = path.split('/')[1:]
                for i in range(len(list_path)):
                    stack.append(list_path.pop())
                vhRootString=path+vhRootString
            if idp==DESTINATION_HIDDEN_FIRST:
                for i in range(len(destPath)):
                    stack.append(destPath.pop())
                vhRootString=self.mapping+vhRootString
        # request.set( "VIRTUAL_HOST_ROOT_PATH", vhRootString )
        request.set( "VIRTUAL_HOST_IDS", addressString )
        return 1

    def getTraversalResponse(self):
        """A hook that the VHF uses to give the host override an
        opportunity to provide a traversal object"""
        # One could also send a destination list here, instead, but at
        # the moment I don't see any advantage of it over the
        # DESTINATION_SHOWN_... options, and I don't feel like
        # building it out just because I can.  The feature of
        # traversal that allows you to include lists like this as a
        # return from a __bobo_traverse__ seems better suited to
        # as-needed redirect-type objects that are actually placed in
        # a folder, rather than a global before-traversal path
        # rewriting tool like the VHF...
        return aq_base(self)


#    security.declarePublic( 'index_html' )
#    def index_html(self, REQUEST=None, RESPONSE=None):
#        """ tell the world... """
#        return self.__bobo_traverse__( REQUEST, 'index_html' )
#        try:
#            obj=self.restrictedTraverse(self.mapping+"/"+name)
#            request.setAliasName(self.getId())
#            if not obj.__doc__: obj.__doc__="dummy"
#            return aq_base(obj).__of__(self)
#        except 'Unauthorized':
#            raise
#        except:
#            return None # I could raise other stuff too I guess...not sure...


    def __bobo_traverse__(self, request, name):
        '''Traversing'''
        # first we need to see if the traversal is within the alias's
        # normal location (within a host, acting like a normal
        # FakeProduct) or if it is somewhere else (acting as a
        # redirector).  Because we use location as the means of
        # determining behavior, trying to mask an alias with an alias
        # is not gonna work too well.  Doesn't seem like too much of a
        # problem.
        ######################################
        ###############parents=request.PARENTS
        ## The above line causes major security problems here, yet works in the VHF.  Why????

        if name=="": name="index_html"

        isNormal=not self.getStatus()
        if not isNormal:
            parent=aq_parent(self)
            while parent and IAlias.IAliasHostOverrideContainer.isImplementedBy(parent):
                if IHost.IHost.isImplementedBy(parent):
                    isNormal=1
                    break
                parent=aq_parent(parent)
        if isNormal:
            x= self.getAcquiredItemById(name)
            if x: return x
            if hasattr(self, name): return getattr(self, name)
            return None
        # ok...we're not in Kansas; time to do tricks.
        # 1) handle ".."
        if name=="..": # will we ever be called with this?  I think so
            o=getattr(self, 'aq_parent', utils.marker)
            if o is not utils.marker:
                securityManager=getSecurityManager()
                if (not securityManager.validate(
                    self, self,name, o)):
                    raise 'Unauthorized', name
                return o
            return None
        # 2) see if it is a child alias
        for child in self.getAcquiredItems():
            if child.getStatus():
                if child.getId()==name:
                    return child.__of__(self)
        # 3) see if it is an active alias
        if self.getForceAlias()==FORCE_ACTIVE_ALIAS:
        #try:
        #raise "eek", (self.mapping+"/"+name)
            address=self.mapping
            if not address or address[-1] != "/": address+="/"
            address+=name # retainPathAfterAlias handled in activateOverride,btw
            try:
                obj=self.restrictedTraverse(address)
                request.setAliasName(self.getId())
                return aq_base(obj).__of__(self)
            except LookupError, AttributeError: # LookupError includes IndexError and KeyError
                pass # because we then want to try the last step...
        # 4) see if it can be acquired
        o=getattr(self, name, utils.marker)
        if o is not utils.marker:
            securityManager=getSecurityManager()
            if not securityManager.validate( self, None, name, o):
                raise 'Unauthorized', name
            request.setAliasName(self.getId())
            return o
        else: return None

    security.declarePublic( 'getVirtualEquivalentPath' )
    def getVirtualEquivalentPath(self, host, pathFromRoot, URLFromRoot):
        """Given a string of the host and a path (list) and URL
        (string) from the Zope root, tries to return a path from the
        virtual root that will use this override or one of its
        children to reach the same location."""
        map=self.getMapping()
        if map==URLFromRoot:
            return self.getFullPath()
        if self.getRetainPathAfterAlias() and map+"/"==URLFromRoot[:len(map)+1]:
            return self.getFullPath()+URLFromRoot[:len(map)+1].split("/")
        for child in self.getAcquiredItems():
            if child.getStatus():
                childpath=child.getVirtualEquivalentPath(host, pathFromRoot, URLFromRoot)
                if childpath:
                    return self.getFullPath()+childpath
        return None

    security.declarePublic( 'getIncludeDestinationPathOptions' )
    def getIncludeDestinationPathOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include in acquisition"),\
            (DESTINATION_HIDDEN_FIRST,"Hide (before virtual root, if present)"),\
            (DESTINATION_HIDDEN_LAST,"Hide (after virtual root, if present)"),\
            (DESTINATION_SHOWN_FIRST,"Show (before alias parents, if present)"),\
            (DESTINATION_SHOWN_LAST,"Show (after alias parents, if present)")\
            #,\
            # (DESTINATION_INSERTED_DYNAMICALLY,"Insert on-the-fly") # requires passive alias.  This functionality has not been done, and may not be..
            )
    security.declarePublic( 'getForceAliasOptions' )
    def getForceAliasOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not force alias in path"),\
            (FORCE_PASSIVE_ALIAS,"Force passive alias in path"),\
            (FORCE_ACTIVE_ALIAS,"Force active alias in path")\
            )
    security.declarePublic( 'getIncludeParentsOptions' )
    def getIncludeParentsOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include in aquisition path"),\
            (INCLUDE_PARENTS,"Include in aquisition path")\
            )
    security.declarePublic( 'getIncludeVirtualRootOptions' )
    def getIncludeHostRootOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include virtual root in aquisition path"),\
            (INCLUDE_HOST_ROOT,"Include virtual root in aquisition path")\
            )
    def getRetainPathAfterAliasOptions(self):
        """returns options for UI"""
        return (\
            (0,"Delete path after alias"),\
            (RETAIN_PATH_AFTER_ALIAS,"Include path after alias")\
            )
    def getSimpleOptions(self):
        """returns simple UI options"""
        return (
    (0,
    "Alias is not in use"),
    (FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST+INCLUDE_PARENTS,
    "Alias appears to be an actual folder, with acquisition enabled for the paths both to the destination folder and to the alias parent location"),
    (FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+INCLUDE_PARENTS,
    "Alias appears to be an actual folder, acquired by the path to the alias parent location (and not the actual destination path)"),
    (DESTINATION_SHOWN_FIRST,
    "Alias makes no pretense to be anything other than a temporary redirect (useful for images, etc.), without any connection to the virtual host root or parents"),
    (INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST,
    "Alias uses standard acquisition to find the destination path so that both the virtual host root and the full path to the destination are available"),
    (INCLUDE_HOST_ROOT+DESTINATION_SHOWN_LAST+INCLUDE_PARENTS,
    "Alias uses standard acquisition to find the destination path so that both the path to the alias parents and the full path to the destination are available")
    )

InitializeClass(Alias)
