
# standard issue modules
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent, aq_base
import re, OFS.PropertyManager

# my own

from Products.Fakers import ZFakeFolder, ZFakeProduct
import Alias, IAlias, IHost, utils

class Host(ZFakeFolder.ZFakeFolder, ZFakeProduct.ZFakeProduct, OFS.PropertyManager.PropertyManager):

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    meta_type="Virtual Host"
    addable_classes=(Alias.Alias,)
    icon="misc_/VirtualHostFolder/host"

    __implements__= IAlias.IAliasHostOverrideContainer, \
                    IHost.IHost, \
                    IHost.IModifiableHost, \
                    IHost.IModifiableRegexHost

    manage_options=(
        (
        {'label':'Configuration', 'action':'manage_main',
        'help':('VirtualHostFolder','VHF_Configure.stx')},
        )
        +OFS.PropertyManager.PropertyManager.manage_options
        +ZFakeFolder.ZFakeFolder.manage_options
        )

    _properties=(
        {'id':'title', 'type': 'string', 'mode': 'w'},\
        #{'id':'CMF_manager_skins', 'type': 'tokens', 'mode': 'w'},\
        #{'id':'CMF_owner_skins', 'type': 'tokens', 'mode': 'w'},\
        #{'id':'CMF_default_skin', 'type': 'string', 'mode': 'w'}\
        )

    ################################### delete these probably ##################
    #CMF_manager_skins=CMF_owner_skins=[]
    #CMF_default_skin=""
    ############################################################################

    # work with the domain list is always case insensitive
    # work with the domainRE is case insensitive by default but modifiable
    ## and now we need UI methods

    security.declareProtected( 'View management screens', 'manage_main' )
    manage_main = DTMLFile('dtml/HostManage_main', globals())

    security.declareProtected( 'View management screens', 'manage_host' )
    def manage_host(self, domainList=[], virtualRoot="", domainRE="", domainREIgnoreCase=0, virtualRootTemplate="", REQUEST=None):
        """the main management function.  The mfunc uses the optional
        messaging system: if a callable object is passed to the various
        worker-bee functions, they will call the function with messaging
        information.  In this case, we are having the worker-bee functions
        append to a messages array that we then return to the caller."""
        messages=[]
        mfunc=messages.append
        self.delDomainsFromList([x for x in self.getDomainList() if x not in domainList], mfunc)
        self.addDomainsToList([x for x in domainList if x not in self.getDomainList()], mfunc)
        if virtualRoot != self.getVirtualRoot():
            self.setVirtualRoot(virtualRoot, mfunc)
        if domainRE!=self.getDomainREComponents()[0] or ((domainREIgnoreCase and 1 or 0) ^ (self.getDomainREComponents()[1] and 1 or 0)):
            self.setDomainRE(domainRE, ignorecase=domainREIgnoreCase, mfunc=mfunc)
        if virtualRootTemplate != self.getVirtualRootTemplate():
            self.setVirtualRootTemplate(virtualRootTemplate, mfunc)
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, messages=messages)
        return messages



    ## the non-UI basic methods

    # _aliasList=() # added for backwards compatibility
    def __init__(self, id, title=""):
        """yup, it's an init."""
        ZFakeFolder.ZFakeFolder.__init__(self, id, title)
        self._virtualRoot="/"
        self._domainList=()
        self._virtualRootTemplate=""
        self._domainRE=None
        self._aliasList=() # tuple of Alias objects

        ## the setters and getters.

    security.declareProtected( 'Manage properties', 'setVirtualRoot' )
    def setVirtualRoot(self, virtualRoot="/", mfunc=None):
        """sets the static virtual root.  Uses optional message system (mfunc)."""
        if not virtualRoot: virtualRoot="/"
        virtualRoot=utils.cleanPath.match(virtualRoot).group(1) # eliminates final slash if present
        self._virtualRoot = self._check_path(virtualRoot, mfunc) and virtualRoot or self._virtualRoot
        self._p_changed = 1
        if mfunc and self._virtualRoot==virtualRoot:
            if self._virtualRoot=="/":
                mfunc("Virtual host root set to Zope root")
            else:
                mfunc("Virtual host root changed")
        return self._virtualRoot==virtualRoot

    security.declarePublic( 'getVirtualRoot' )
    def getVirtualRoot(self):
        """get the static path."""
        return self._virtualRoot

    security.declarePublic( 'getVirtualRootWithRegex' ) # magic
    def getVirtualRootWithRegex(self, matchObj=None, matchObj2=None):
        """Call it with no arguments, and you will get the static
        path.  Call it with a regular expression match object or two
        (designed for given case vs. lowercase), and it will try to
        return the expanded (filled-in) version of the dynamic virtual
        root template; if the template does not exist or the expanded
        path does not resolve to a valid Zope folder, the static path
        is returned."""

        if matchObj and self._virtualRootTemplate:
            dynamicRoot=matchObj.expand(self._virtualRootTemplate)
            if self._check_path(dynamicRoot): return dynamicRoot
            if matchObj2: # experimental lowercase
                dynamicRoot=matchObj2.expand(self._virtualRootTemplate)
                if self._check_path(dynamicRoot): return dynamicRoot
        return self._virtualRoot

    security.declareProtected( 'Manage properties', 'addDomainsToList' )
    def addDomainsToList(self, domains, mfunc=None):
        """adds domains to list; uses optional message system"""
        domains=self._clean_domainList(domains) # shape into a lowercase tuple, remove blanks
        ln=len(self._domainList)
        self._domainList+=tuple([x for x in domains if x not in self._domainList and self._check_domain(x, mfunc)])
        ln=len(self._domainList)-ln
        if ln:
            self._p_changed = 1
            if mfunc:mfunc("%d domain%s added to domain list" % (ln, ln!=1 and "s" or ""))
        return ln

    security.declareProtected( 'Manage properties', 'delDomainsFromList' )
    def delDomainsFromList(self, domains, mfunc=None):
        """deletes domains from list; uses optional message system"""
        domains=self._clean_domainList(domains) # shape into a lowercase tuple
        ln=len(self._domainList)
        self._domainList=tuple([x for x in self._domainList if x not in domains])
        ln=ln-len(self._domainList)
        if ln:
            self._p_changed = 1
            if mfunc:mfunc("%d domain%s deleted from domain list" % (ln, ln!=1 and "s" or ""))
        return ln

    security.declarePublic( 'getDomainList' )
    def getDomainList(self):
        """returns an unsorted domain list"""
        return self._domainList

    security.declarePublic( 'getSortedDomainList' )
    def getSortedDomainList(self):
        """returns a sorted version of the domain list"""
        x=list(self._domainList)
        x.sort()
        return x

    security.declareProtected( 'Manage properties', 'setVirtualRootTemplate' )
    def setVirtualRootTemplate(self, tmpl, mfunc=None):
        """sets the virtual root template; supports the optional
        messaging system."""
        if not tmpl:
            if mfunc:mfunc("Virtual root template deleted.")
            self._virtualRootTemplate=""
            self._p_changed = 1
            return 1
        virtualRoot=utils.cleanPath.match(tmpl).group(1) # eliminates final slash if present
        if self.isUsefulTemplate.search(tmpl):
            self._virtualRootTemplate=tmpl
            if mfunc:mfunc("Virtual root template changed")
            self._p_changed = 1
            return 1
        if mfunc:mfunc(r"Virtual rule template not set: the template must use back references such as \1 or else it is not a dynamic template.  Just set the standard virtual root instead.  If you want the regular expression to go to a different standard virtual root, consider creating a new host.")
        return 0

    security.declarePublic( 'getVirtualRootTemplate' )
    def getVirtualRootTemplate(self):
        """returns the unprocessed vitual root template"""
        return self._virtualRootTemplate

    security.declareProtected( 'Manage properties', 'setDomainRE' )
    def setDomainRE(self, domainRE, ignorecase=1, mfunc=None):
        """sets the domain regular expression.  Pass in anything that evaluates
        to Boolean true in ignorecase to set it to true.  No other flags are
        available (although they could be added, I didn't feel they would be
        useful).  The messages array is an optional human-readable
        messaging system described in the doc string for _check_path below: feel
        free to ignore."""
        if domainRE:
            test=0
            try:
                test=re.compile(domainRE, ignorecase and re.I or 0)
            except (SyntaxError, re.error), e:
                # get_transaction().abort()
                if mfunc:mfunc("The domain regular expression could not be changed. The most likely error is a syntax error in your regular expression.  Python reports the following: %s" % "; ".join(getErrorText(e)))
                return 0
            if mfunc:mfunc("The domain regular expression is %s" % (self._domainRE and "changed" or "added"))
            self._domainRE=test
        else:
            self._domainRE=None
            if mfunc:mfunc("The domain regular expression is deleted")
        self._p_changed = 1
        return 1

    security.declarePublic( 'getDomainRE' )
    def getDomainRE(self):
        """returns the current domain regular expression *object*"""
        return self._domainRE

    security.declareProtected( 'Manage properties', 'getDomainREComponents' )
    def getDomainREComponents(self):
        """returns the current domain regular expression and the ignorecase
        flag setting.  If no current domain regular expression is set, returns an empty string and the default 'on' ignorecase setting."""
        return self._domainRE and (self._domainRE.pattern, self._domainRE.flags and 1) or ("", 1)

    security.declarePrivate('_matchDomainRE') # unnecessary
    def _matchDomainRE(self, domain):
        """silently handles absence of RE; included in this group because it
        directly accesses the _domainRE rather than using the getter.  Perhaps
        a weird, misguided combination of anal and faux-speed-conscious, at
        worst."""
        return self._domainRE and self._domainRE.match(domain) or None

    ## the other basic method(s)
    security.declarePublic( 'testHost' )
    def testHost(self, host, stack, VHFid, request):
        host=host.strip()
        lowhost=host.lower()
        if lowhost in self.getDomainList(): return self.activateHost(host, stack, VHFid, request)
        res=self._matchDomainRE(host)
        if res:
            res2= (host != lowhost) and self.getVirtualRootTemplate() and self._matchDomainRE(lowhost) or 0
            return self.activateHost(host, stack, VHFid, request, res, res2)
        return None

    security.declarePublic( 'activateOverride' )
    def activateHost(self, host, stack, VHFid, request, res=None, res2=None):
        path=self.getVirtualRootWithRegex(res, res2)
        request.set( "CANONICAL_VIRTUAL_HOST_ROOT_PATH", path )
        aliasList=self.getAcquiredItems()
        if aliasList:
            stackString=""
            for loc in stack:
                stackString="/"+loc+stackString
            for alias in aliasList:
                if alias.testOverride(host, stack, stackString):
                    return alias.activateOverride(stack, VHFid, request, self.getVirtualRoot(), [self.getId()]) # always use standard virtual root (not expanded) when dealing with aliases
        # set stack with no alias
        # request.set( "VIRTUAL_HOST_ROOT_PATH", path )
        request.set( "VIRTUAL_HOST_IDS", self.getId() )
        list_path = path.split('/')[1:]
        stack.append('/') # the flag that bobo_traverse looks for
        stack.append(VHFid)
        for i in range(len(list_path)):
            stack.append(list_path.pop())
        return 1


    security.declareProtected( 'Manage properties', 'checkParentsAndAliasName' )
    def checkParentsAndAliasName(self, parent, aliasName, caller, mfunc=None):
        aliasList=self.getItems()
        for aliasObj in aliasList:
            if aliasObj != caller:
                objParent=aliasObj.getParent()
                if objParent==parent:
                    objName=aliasObj.getAliasName()
                    if objName==aliasName:
                        if mfunc: mfunc("An alias with the identical address (parent and name) already exists in the parent folder.")
                        return 0
                elif objParent+"/" == parent[:(len(objParent)+1)]:
                    if mfunc: mfunc("You appear to be trying to place an alias within another alias.  To do this, actually add an alias within the contents of the parent alias folder, rather than specifying it in the parent field.")
                    return 0
                elif parent+"/" == objParent[:(len(parent)+1)]:
                    if mfunc: mfunc("An already existing alias, %s, would be masked by this one if you used this parent.  If the other alias needs to exist within this one, you may need to delete it, change the parent in this alias to what you want, and then add the existing alias within this one.""" % aliasObj.getId())
                    return 0
        fullPath=self.getVirtualRoot()+parent
        try:
            parentObj=self.unrestrictedTraverse(fullPath)
        except:
            if mfunc: mfunc("The parent must exist within the standard virtual root.  %s does not exist." % fullPath)
            return 0
        return 1

    security.declarePrivate('_check_domain') # unnecessary
    def _check_domain(self, domain, mfunc=None):
        """This is a plug to put in code that rejects domains.  At the
        moment, all it does is try to make sure the domain is not a numeric IP
        address.  That's probably all it will ever need to do too, but hey, here
        it is."""
        if utils.isIPAddress.match(domain):
            if mfunc:mfunc("%s : IP addresses are not mappable" % domain)
            return 0
        return 1

    security.declarePrivate('_clean_domainList') # unnecessary
    def _clean_domainList(self, domains, mfunc=None):
        """a utility function.  converts all of the domain names to lower
        case, which is convenient for the domain list functionality, and does
        whatever else needs doing to the list to shape it up."""
        return [x.lower().strip() for x in domains if x and not x.isspace() ]



InitializeClass(Host)
