from Products.Fakers import IZFakers
# from Interface import Base

class IHost( IZFakers.IZFakeProduct ):
    """A class to represent one virtual host, presumable as held by a Virtual Host Folder."""

    def testHost(self, host, stack, VHFid, request):
        """Given the full host (e.g. 'test.my.site.com'), the
        request's stack (or reasonable testing facsimile), the id of
        the parent Virtual Host Folder, and the request object (or
        reasonable testing facsimile), either return no match (0 or
        other Boolean false) to inform the VHF that this is not the
        correct Host for the job, or call activateHost and return a
        success message (1 or other Boolean true)."""

    def activateHost(self, host, stack, VHFid, request, res=None, res2=None):
        """Given the same arguments as testHost plus one or two
        optional regex match objects, modify the request as necessary
        and return a success message."""

class IModifiableHost( IHost ):
    """setters and getters for a basic modifiable host implementation"""

    def setVirtualRoot(self, virtualRoot="/", mfunc=None):
        """sets the static virtual root.  Uses optional message system (mfunc)."""

    def getVirtualRoot(self):
        """get the static virtual root."""

    def addDomainsToList(self, domains, mfunc=None):
        """adds domains (a list) to the current list of matching
        domains for this virtual host.  Domains are strictly 1-to-1
        matching--no pattern matching of any sort (see the additional
        regex interface below for that).  Calls optional mfunc with
        any pertinent messages."""

    def delDomainsFromList(self, domains, mfunc=None):
        """deletes list of domains from current list of matching
        domains for this host.  Calls optional mfunc with any
        pertinent messages."""

    def getDomainList(self):
        """Returns the current list of domains for this virtual
        host."""

class IModifiableRegexHost ( IModifiableHost ):
    """An extension to the IModifiableHost interface that adds regex
    capability."""

    def getVirtualRootWithRegex(self, matchObj=None, matchObj2=None):
        """a magic function.  Call it with no arguments, and you will
        get the static path.  Call it with a regular expression match
        object or two (designed for given case vs. lowercase), and it
        will try to return the expanded (filled-in) version of the
        dynamic virtual root template; if the template does not exist
        or the expanded path does not resolve to a valid Zope folder,
        the static path is returned."""

    def setVirtualRootTemplate(self, tmpl, mfunc=None):
        """sets the virtual root template; supports the optional
        messaging system."""
        
    def getVirtualRootTemplate(self):
        """returns the unprocessed vitual root template"""

    def setDomainRE(self, domainRE, ignorecase=1, mfunc=None):
        """sets the domain regular expression.  Pass in anything that
        evaluates to Boolean true in ignorecase to set it to true.  No
        other flags are available (although they could be added, I
        didn't feel they would be useful).  The messages array is an
        optional human-readable messaging system: feel free to
        ignore."""

    def getDomainRE(self):
        """returns the current domain regular expression *object*"""

    def getDomainREComponents(self):
        """returns the current domain regular expression and the
        ignorecase flag setting.  If no current domain regular
        expression is set, returns an empty string and the default
        'on' ignorecase setting."""
